﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class LevelManager : MonoBehaviour {

    public Texture2D[] levels;
    int level = -1;
    public Transform cursor;
    public float smoothing = .1f;
    Vector2 pos;
    Camera cam;
    bool playing = false;
    bool beatGame = true;
    bool beatLevel = false;
    public float distanceToCenter = 25;
    public float distanceToEdge = 25;
    public Text[] menu;
    public Text[] next;
    public Text[] restart;
    float restartAlpha = 0;
    public AnimationCurve hideCurve;
    public AnimationCurve showCurve;
    Text[] fading;
    bool animating = false;
    float started = 0;
    float value = 0;
    bool canClick = true;

    public string onLevelEvent;

    public CCReflectTexture levelPreview;
    public Material mat;
    public Material orig;

	// Use this for initialization
	void Start () {
        pos = Input.mousePosition;
        cam = GameObject.FindObjectOfType<Camera>();
        Cursor.visible = false;
        mat.CopyPropertiesFromMaterial(orig);
	}
	
	// Update is called once per frame
	void Update () {
        pos = Vector2.Lerp(pos, new Vector2(Input.mousePosition.x, Input.mousePosition.y), smoothing);
        cursor.transform.position = cam.ScreenToWorldPoint(new Vector3(pos.x,pos.y,0));
        if (Input.GetKeyDown(KeyCode.Escape))
            Cursor.visible = true;
        if(Input.GetMouseButtonDown(0))
            Cursor.visible = false;

        if (beatGame)
            return;

        if (beatLevel)
            value =  Mathf.Lerp(value, 1, .1f);
        else
            value = levels[level].GetPixel(Mathf.FloorToInt(pos.x), Mathf.FloorToInt(pos.y)).r;

        started = Mathf.Lerp(started, 1, .1f);
        if (!playing && !beatLevel)
        {
            restartAlpha = Mathf.Lerp(restartAlpha, .5f, .1f);
        }
        else
        {
            restartAlpha = Mathf.Lerp(restartAlpha, 0f, .1f);
        }
        Fade(restart, restartAlpha);

        mat.SetVector("_Mouse", new Vector4((pos.x / Screen.width - .5f) * started, (pos.y / Screen.height - .5f) * started, Mathf.Min(1 - restartAlpha * 2, value), mat.GetVector("_Mouse").w));

        if (Vector2.Distance(pos,new Vector2(Screen.width * .5f, Screen.height * .5f))<distanceToCenter&&!beatLevel)
        {
            playing = true;
        }
        if (playing)
        {
            if (value == 0)
            {
                playing = false;
            }
            else if (pos.x < distanceToEdge || pos.y < distanceToEdge || pos.x > Screen.width - distanceToEdge || pos.y > Screen.height - distanceToEdge ||Input.GetKeyDown(KeyCode.Space))
            {
                FinishLevel();
            }
        }
        
	}

    void FinishLevel()
    {
        beatLevel = true;
        playing = false;
        fading = next;
        StartCoroutine(Utils.AnimationCoroutine(showCurve, .5f, Fade));
    }
    

    public void ClickCenter() {
        if (!canClick)
            return;
        if (beatGame)
        {
            fading = menu;
            StartCoroutine(Utils.AnimationCoroutine(hideCurve, .5f, Fade, StartGame));
        }
        else if(beatLevel)
        {
            NextLevel();
        }
    }

    void StartGame()
    {
        NextLevel();
        beatGame = false;
    }

    void Fade(float a)
    {
        a *= 5f;
        foreach (Text t in fading)
        {
            Color c = t.color;
            c.a = a;
            t.color = c;
        }
    }
    void Fade(Text[] elements, float a)
    {
        a *= .5f;
        foreach (Text t in elements)
        {
            Color c = t.color;
            c.a = a;
            t.color = c;
        }
    }

    void NextLevel()
    {
        level++;
        beatLevel = false;
        Messenger.Broadcast(onLevelEvent);
        Messenger.Broadcast(onLevelEvent+level);
        if (level < levels.Length)
        {
            levelPreview.SetValue(levels[level]);
            fading = next;
            StartCoroutine(Utils.AnimationCoroutine(hideCurve, .5f, Fade));
        }
        else
        {
            playing = false;
            level = -1;
            canClick = false;
            beatGame = true;
            beatLevel = true;
            fading = menu;
            StartCoroutine(Utils.AnimationCoroutine(showCurve, .5f, Fade, Remenu));
            StartCoroutine(Utils.AnimationCoroutine(showCurve, .5f, ResetMat));
        }
    }

    void Remenu()
    {
        canClick = true;
    }

    void ResetMat(float a)
    {
        mat.SetVector("_x", Vector4.Lerp(mat.GetVector("_x"), orig.GetVector("_x"), a));
        mat.SetVector("_Shape", Vector4.Lerp(mat.GetVector("_Shape"), orig.GetVector("_Shape"), a));
        mat.SetVector("_Mouse", Vector4.Lerp(mat.GetVector("_Mouse"), orig.GetVector("_Mouse"), a));
        mat.SetColor("_Color1", Vector4.Lerp(mat.GetColor("_Color1"), orig.GetColor("_Color1"), a));
        mat.SetColor("_Color2", Vector4.Lerp(mat.GetColor("_Color2"), orig.GetColor("_Color2"), a));
        mat.SetColor("_Color3", Vector4.Lerp(mat.GetColor("_Color3"), orig.GetColor("_Color3"), a));
    }
}
